@@ -24,7 +24,10 @@
 
 #include "hexchat.h"
 
+#define LANGUAGES_LENGTH 52
+
 extern char *xdir;
+extern const char const *languages[LANGUAGES_LENGTH];
 
 char *cfg_get_str (char *cfg, const char *var, char *dest, int dest_len);
 int cfg_get_bool (char *var);
@@ -34,7 +37,11 @@ int cfg_put_int (int fh, int value, char *var);
 int cfg_get_color (char *cfg, char *var, int *r, int *g, int *b);
 int cfg_put_color (int fh, int r, int g, int b, char *var);
 char *get_xdir (void);
-void load_config (void);
+int check_config_dir (void);
+void load_default_config (void);
+int make_config_dirs (void);
+int make_dcc_dirs (void);
+int load_config (void);
 int save_config (void);
 void list_free (GSList ** list);
 void list_loadconf (char *file, GSList ** list, char *defaultconf);
@@ -43,6 +50,7 @@ void list_addentry (GSList ** list, char *cmd, char *name);
 int cmd_set (session *sess, char *tbuf, char *word[], char *word_eol[]);
 int hexchat_open_file (char *file, int flags, int mode, int xof_flags);
 FILE *hexchat_fopen_file (const char *file, const char *mode, int xof_flags);
+
 #define XOF_DOMODE 1
 #define XOF_FULLPATH 2
 
@@ -69,4 +77,6 @@ struct prefs
 #define TYPE_INT 1
 #define TYPE_BOOL 2
 
+#define HEXCHAT_SOUND_DIR "sounds"
+
 #endif
