@@ -102,7 +102,9 @@ main (int argc, char **argv)
 	gchar *path;
 	GError *error = NULL;
 
+#if ! GLIB_CHECK_VERSION (2, 36, 0)
 	g_type_init ();
+#endif
 
 	connection = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
 	if (connection == NULL) {
