@@ -30,21 +30,6 @@
 
 #include "fe-gtk.h"
 
-#include <gtk/gtkalignment.h>
-#include <gtk/gtkcellrenderertext.h>
-#include <gtk/gtkcheckbutton.h>
-#include <gtk/gtkcombobox.h>
-#include <gtk/gtkentry.h>
-#include <gtk/gtkhbox.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtkliststore.h>
-#include <gtk/gtkscrolledwindow.h>
-#include <gtk/gtkspinbutton.h>
-#include <gtk/gtkstock.h>
-#include <gtk/gtktable.h>
-#include <gtk/gtktreeselection.h>
-#include <gtk/gtkvbox.h>
-#include <gtk/gtkvseparator.h>
 #include <gdk/gdkkeysyms.h>
 
 #include "../common/hexchat.h"
@@ -58,10 +43,8 @@
 #include "maingui.h"
 #include "menu.h"
 
-
 #include "custom-list.h"
 
-
 enum
 {
 	COL_CHANNEL,
@@ -549,6 +532,7 @@ chanlist_minusers (GtkSpinButton *wid, server *serv)
 {
 	serv->gui->chanlist_minusers = gtk_spin_button_get_value_as_int (wid);
 	prefs.hex_gui_chanlist_minusers = serv->gui->chanlist_minusers;
+	save_config();
 
 	if (serv->gui->chanlist_minusers < serv->gui->chanlist_minusers_downloaded)
 	{
@@ -570,6 +554,7 @@ chanlist_maxusers (GtkSpinButton *wid, server *serv)
 {
 	serv->gui->chanlist_maxusers = gtk_spin_button_get_value_as_int (wid);
 	prefs.hex_gui_chanlist_maxusers = serv->gui->chanlist_maxusers;
+	save_config();
 }
 
 static void
@@ -644,7 +629,7 @@ chanlist_button_cb (GtkTreeView *tree, GdkEventButton *event, server *serv)
 								chanlist_copytopic, serv);
 
 	chan = chanlist_get_selected (serv, FALSE);
-	menu_addfavoritemenu (serv, menu, chan);
+	menu_addfavoritemenu (serv, menu, chan, FALSE);
 	g_free (chan);
 
 	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 0, event->time);
@@ -742,6 +727,7 @@ chanlist_opengui (server *serv, int do_refresh)
 		if (prefs.hex_gui_chanlist_minusers < 1 || prefs.hex_gui_chanlist_minusers > 999999)
 		{
 			prefs.hex_gui_chanlist_minusers = 5;
+			save_config();
 		}
 
 		serv->gui->chanlist_minusers = prefs.hex_gui_chanlist_minusers;
@@ -752,6 +738,7 @@ chanlist_opengui (server *serv, int do_refresh)
 		if (prefs.hex_gui_chanlist_maxusers < 1 || prefs.hex_gui_chanlist_maxusers > 999999)
 		{
 			prefs.hex_gui_chanlist_maxusers = 9999;
+			save_config();
 		}
 
 		serv->gui->chanlist_maxusers = prefs.hex_gui_chanlist_maxusers;
@@ -760,6 +747,7 @@ chanlist_opengui (server *serv, int do_refresh)
 	serv->gui->chanlist_window =
 		mg_create_generic_tab ("ChanList", tbuf, FALSE, TRUE, chanlist_closegui,
 								serv, 640, 480, &vbox, serv);
+	gtkutil_destroy_on_esc (serv->gui->chanlist_window);
 
 	gtk_container_set_border_width (GTK_CONTAINER (vbox), 6);
 	gtk_box_set_spacing (GTK_BOX (vbox), 12);
