@@ -21,29 +21,6 @@
 #include <stdio.h>
 #include <ctype.h>
 
-#include <gtk/gtkarrow.h>
-#include <gtk/gtktogglebutton.h>
-#include <gtk/gtkhbox.h>
-#include <gtk/gtkvbox.h>
-#include <gtk/gtkeventbox.h>
-#include <gtk/gtkentry.h>
-#include <gtk/gtkhpaned.h>
-#include <gtk/gtkvpaned.h>
-#include <gtk/gtkframe.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtkmenuitem.h>
-#include <gtk/gtkprogressbar.h>
-#include <gtk/gtkscrolledwindow.h>
-#include <gtk/gtkstock.h>
-#include <gtk/gtktable.h>
-#include <gtk/gtknotebook.h>
-#include <gtk/gtkimage.h>
-#include <gtk/gtkmessagedialog.h>
-#include <gtk/gtkcheckmenuitem.h>
-#include <gtk/gtkcheckbutton.h>
-#include <gtk/gtkbbox.h>
-#include <gtk/gtkvscrollbar.h>
-
 #include "../common/hexchat.h"
 #include "../common/fe.h"
 #include "../common/server.h"
@@ -55,6 +32,7 @@
 #include "../common/url.h"
 #include "../common/util.h"
 #include "../common/text.h"
+#include "../common/chanopt.h"
 
 #include "fe-gtk.h"
 #include "banlist.h"
@@ -71,7 +49,6 @@
 #include "xtext.h"
 
 #ifdef USE_GTKSPELL
-#include <gtk/gtktextview.h>
 #include <gtkspell/gtkspell.h>
 #endif
 
@@ -267,7 +244,8 @@ fe_set_tab_color (struct session *sess, int col)
 			chan_set_color (sess->res->tab, newdata_list);
 
 			if (chan_is_collapsed (sess->res->tab)
-				&& !(server_sess->msg_said || server_sess->nick_said))
+				&& !(server_sess->msg_said || server_sess->nick_said)
+				&& !(server_sess == current_tab))
 			{
 				server_sess->new_data = TRUE;
 				server_sess->msg_said = FALSE;
@@ -282,7 +260,9 @@ fe_set_tab_color (struct session *sess, int col)
 			sess->nick_said = FALSE;
 			chan_set_color (sess->res->tab, newmsg_list);
 			
-			if (chan_is_collapsed (sess->res->tab) && !server_sess->nick_said)
+			if (chan_is_collapsed (sess->res->tab) 
+				&& !server_sess->nick_said
+				&& !(server_sess == current_tab))
 			{
 				server_sess->new_data = FALSE;
 				server_sess->msg_said = TRUE;
@@ -297,7 +277,7 @@ fe_set_tab_color (struct session *sess, int col)
 			sess->nick_said = TRUE;
 			chan_set_color (sess->res->tab, nickseen_list);
 
-			if (chan_is_collapsed (sess->res->tab))
+			if (chan_is_collapsed (sess->res->tab) && !(server_sess == current_tab))
 			{
 				server_sess->new_data = FALSE;
 				server_sess->msg_said = FALSE;
@@ -501,7 +481,7 @@ mg_windowstate_cb (GtkWindow *wid, GdkEventWindowState *event, gpointer userdata
 {
 	if ((event->changed_mask & GDK_WINDOW_STATE_ICONIFIED) &&
 		 (event->new_window_state & GDK_WINDOW_STATE_ICONIFIED) &&
-		 prefs.hex_gui_tray_minimize && !hextray_mode () && !unity_mode ())
+		 prefs.hex_gui_tray_minimize && !unity_mode ())
 	{
 		tray_toggle_visibility (TRUE);
 		gtk_window_deiconify (wid);
@@ -1295,7 +1275,7 @@ mg_open_quit_dialog (gboolean minimize_button)
 	gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1),
 										GTK_BUTTONBOX_END);
 
-	if (minimize_button && !hextray_mode () && !unity_mode ())
+	if (minimize_button && !unity_mode ())
 	{
 		button = gtk_button_new_with_mnemonic (_("_Minimize to Tray"));
 		gtk_widget_show (button);
@@ -1558,6 +1538,9 @@ mg_set_guint8 (GtkCheckMenuItem *item, guint8 *setting)
 	/* has the logging setting changed? */
 	if (logging != sess->text_logging)
 		log_open_or_close (sess);
+
+	chanopt_save (sess);
+	chanopt_save_all ();
 }
 
 static void
@@ -1582,7 +1565,10 @@ mg_create_perchannelmenu (session *sess, GtkWidget *menu)
 	mg_perchan_menu_item (_("_Log to Disk"), submenu, &sess->text_logging, prefs.hex_irc_logging);
 	mg_perchan_menu_item (_("_Reload Scrollback"), submenu, &sess->text_scrollback, prefs.hex_text_replay);
 	if (sess->type == SESS_CHANNEL)
+	{
+		mg_perchan_menu_item (_("Strip _Colors"), submenu, &sess->text_strip, prefs.hex_text_stripcolor_msg);
 		mg_perchan_menu_item (_("_Hide Join/Part Messages"), submenu, &sess->text_hidejoinpart, prefs.hex_irc_conf_mode);
+	}
 }
 
 static void
@@ -1594,10 +1580,7 @@ mg_create_alertmenu (session *sess, GtkWidget *menu)
 
 	mg_perchan_menu_item (_("Beep on _Message"), submenu, &sess->alert_beep, prefs.hex_input_beep_chans);
 
-	if (!hextray_mode ())		/*disable this context menu item when HexTray is loaded */
-	{
-		mg_perchan_menu_item (_("Blink Tray _Icon"), submenu, &sess->alert_tray, prefs.hex_input_tray_chans);
-	}
+	mg_perchan_menu_item (_("Blink Tray _Icon"), submenu, &sess->alert_tray, prefs.hex_input_tray_chans);
 
 	mg_perchan_menu_item (_("Blink Task _Bar"), submenu, &sess->alert_taskbar, prefs.hex_input_flash_chans);
 }
@@ -1634,7 +1617,9 @@ mg_create_tabmenu (session *sess, GdkEventButton *event, chan *ch)
 		menu_quick_item (0, 0, menu, XCMENU_SHADED, 0, 0);
 
 		if (sess->type == SESS_CHANNEL)
-			menu_addfavoritemenu (sess->server, menu, sess->channel);
+			menu_addfavoritemenu (sess->server, menu, sess->channel, TRUE);
+		else if (sess->type == SESS_SERVER)
+			menu_addconnectmenu (sess->server, menu);
 	}
 
 	mg_create_icon_item (_("_Detach"), GTK_STOCK_REDO, menu,
@@ -2117,8 +2102,8 @@ mg_create_chanmodebuttons (session_gui *gui, GtkWidget *box)
 	gui->flag_k = mg_create_flagbutton (_("Keyword"), box, "K");
 	gui->key_entry = gtk_entry_new ();
 	gtk_widget_set_name (gui->key_entry, "hexchat-inputbox");
-	gtk_entry_set_max_length (GTK_ENTRY (gui->key_entry), 16);
-	gtk_widget_set_size_request (gui->key_entry, 30, -1);
+	gtk_entry_set_max_length (GTK_ENTRY (gui->key_entry), 23);
+	gtk_widget_set_size_request (gui->key_entry, 115, -1);
 	gtk_box_pack_start (GTK_BOX (box), gui->key_entry, 0, 0, 0);
 	g_signal_connect (G_OBJECT (gui->key_entry), "activate",
 							G_CALLBACK (mg_key_entry_cb), NULL);
@@ -2168,7 +2153,7 @@ mg_dialog_button_cb (GtkWidget *wid, char *cmd)
 
 	auto_insert (buf, sizeof (buf), cmd, 0, 0, "", "", "",
 					 server_get_network (current_sess->server, TRUE), host, "",
-					 current_sess->channel);
+					 current_sess->channel, "");
 
 	handle_command (current_sess, buf, TRUE);
 
@@ -2260,28 +2245,33 @@ static void
 mg_word_clicked (GtkWidget *xtext, char *word, GdkEventButton *even)
 {
 	session *sess = current_sess;
-	int word_type, start, end;
+	int word_type = 0, start, end;
 	char *tmp;
 
-	if (word == NULL)
+	if (word)
 	{
-		if (even->button == 1)		/* left button */
-			mg_focus (sess);
-		return;
+		word_type = mg_word_check (xtext, word);
+		url_last (&start, &end);
 	}
 
-	word_type = mg_word_check (xtext, word);
-	url_last (&start, &end);
-
-	if (even->button == 1 && (even->state & 13) == prefs.hex_gui_url_mod)
+	if (even->button == 1)			/* left button */
 	{
-		switch (word_type)
+		if (word == NULL)
 		{
-		case WORD_URL:
-		case WORD_HOST:
-			word[end] = 0;
-			word += start;
-			fe_open_url (word);
+			mg_focus (sess);
+			return;
+		}
+
+		if ((even->state & 13) == prefs.hex_gui_url_mod)
+		{
+			switch (word_type)
+			{
+			case WORD_URL:
+			case WORD_HOST6:
+			case WORD_HOST:
+				word[end] = 0;
+				fe_open_url (word + start);
+			}
 		}
 		return;
 	}
@@ -2294,6 +2284,8 @@ mg_word_clicked (GtkWidget *xtext, char *word, GdkEventButton *even)
 			userlist_select (sess, word);
 		return;
 	}
+	if (word == NULL)
+		return;
 
 	switch (word_type)
 	{
@@ -2302,6 +2294,7 @@ mg_word_clicked (GtkWidget *xtext, char *word, GdkEventButton *even)
 		menu_middlemenu (sess, even);
 		break;
 	case WORD_URL:
+	case WORD_HOST6:
 	case WORD_HOST:
 		word[end] = 0;
 		word += start;
@@ -2313,6 +2306,8 @@ mg_word_clicked (GtkWidget *xtext, char *word, GdkEventButton *even)
 		menu_nickmenu (sess, even, word, FALSE);
 		break;
 	case WORD_CHANNEL:
+		word[end] = 0;
+		word += start;
 		menu_chanmenu (sess, even, word);
 		break;
 	case WORD_EMAIL:
@@ -2629,7 +2624,7 @@ mg_create_center (session *sess, session_gui *gui, GtkWidget *box)
 	mg_create_textarea (sess, vbox);
 	mg_create_entry (sess, vbox);
 
-	g_idle_add ((GSourceFunc)mg_add_pane_signals, gui);
+	mg_add_pane_signals (gui);
 }
 
 static void
@@ -3087,7 +3082,7 @@ mg_tabwindow_de_cb (GtkWidget *widget, GdkEvent *event, gpointer user_data)
 	GSList *list;
 	session *sess;
 
-	if (prefs.hex_gui_tray_close && !hextray_mode () && !unity_mode () && tray_toggle_visibility (FALSE))
+	if (prefs.hex_gui_tray_close && !unity_mode () && tray_toggle_visibility (FALSE))
 		return TRUE;
 
 	/* check for remaining toplevel windows */
