@@ -26,8 +26,6 @@
 #include "../common/fe.h"
 
 #include <gdk-pixbuf/gdk-pixbuf.h>
-#include <gdk-pixbuf/gdk-pixdata.h>
-#include <gtk/gtkstock.h>
 
 #include "../pixmaps/inline_pngs.h"
 
