@@ -29,7 +29,6 @@
 #include "pixmaps.h"
 #include "maingui.h"
 #include "menu.h"
-#include <gtk/gtk.h>
 
 #ifndef WIN32
 #include <unistd.h>
@@ -168,7 +167,8 @@ fe_tray_set_balloon (const char *title, const char *text)
 
 	/* no balloons if the window is focused */
 	ws = tray_get_window_status ();
-	if (ws == WS_FOCUSED)
+	if ((prefs.hex_away_omit_alerts && hexchat_get_info(ph, "away")) ||
+		(prefs.hex_gui_focus_omitalerts && ws == WS_FOCUSED))
 		return;
 
 	/* bit 1 of flags means "no balloons unless hidden/iconified" */
@@ -180,13 +180,23 @@ fe_tray_set_balloon (const char *title, const char *text)
 		return;
 
 #ifdef USE_LIBNOTIFY
+	static int notify_text_strip_flags = STRIP_ALL;
 	NotifyNotification *notification;
 	char *notify_text, *notify_title;
 
 	if (!notify_is_initted())
+	{
 		notify_init(PACKAGE_NAME);
 
-	notify_text = strip_color (text, -1, STRIP_ALL|STRIP_ESCMARKUP);
+		GList* server_caps = notify_get_server_caps ();
+		if (g_list_find_custom (server_caps, "body-markup", (GCompareFunc)strcmp))
+		{
+			notify_text_strip_flags |= STRIP_ESCMARKUP;
+		}
+		g_list_free_full (server_caps, g_free);
+	}
+
+	notify_text = strip_color (text, -1, notify_text_strip_flags);
 	notify_title = strip_color (title, -1, STRIP_ALL);
 
 	notification = XC_NOTIFY_NEW (notify_title, notify_text, HEXCHATSHAREDIR "/icons/hicolor/scalable/apps/hexchat.svg", NULL);
@@ -414,11 +424,7 @@ tray_toggle_visibility (gboolean force_hide)
 	if (!win)
 		return FALSE;
 
-#if GTK_CHECK_VERSION(2,20,0)
 	if (force_hide || gtk_widget_get_visible (GTK_WIDGET (win)))
-#else
-	if (force_hide || GTK_WIDGET_VISIBLE (win))
-#endif
 	{
 		if (prefs.hex_gui_tray_away)
 			hexchat_command (ph, "ALLSERV AWAY");
@@ -694,7 +700,7 @@ tray_hilight_cb (char *word[], void *userdata)
 	/*if (tray_status == TS_HIGHLIGHT)
 		return HEXCHAT_EAT_NONE;*/
 
-	if (prefs.hex_input_tray_hilight && (!prefs.hex_away_omit_alerts || tray_find_away_status () != 1))
+	if (prefs.hex_input_tray_hilight)
 	{
 		tray_set_flash (ICON_HILIGHT);
 
@@ -708,8 +714,8 @@ tray_hilight_cb (char *word[], void *userdata)
 								tray_hilight_count, word[1], hexchat_get_info (ph, "channel"));
 	}
 
-	if (prefs.hex_input_balloon_hilight && (!prefs.hex_away_omit_alerts || tray_find_away_status () != 1))
-		tray_set_balloonf (word[2], _(DISPLAY_NAME": Highlighted message from: %s (%s)"),
+	if (prefs.hex_input_balloon_hilight)
+		tray_set_balloonf (word[2], _("Highlighted message from: %s (%s)"),
 								 word[1], hexchat_get_info (ph, "channel"));
 
 	return HEXCHAT_EAT_NONE;
@@ -720,21 +726,21 @@ tray_message_cb (char *word[], void *userdata)
 {
 	if (/*tray_status == TS_MESSAGE ||*/ tray_status == TS_HIGHLIGHT)
 		return HEXCHAT_EAT_NONE;
-
-	if (prefs.hex_input_tray_chans && (!prefs.hex_away_omit_alerts || tray_find_away_status () != 1))
+		
+	if (prefs.hex_input_tray_chans)
 	{
 		tray_set_flash (ICON_MSG);
 
 		tray_pub_count++;
 		if (tray_pub_count == 1)
-			tray_set_tipf (_(DISPLAY_NAME": New public message from: %s (%s)"),
+			tray_set_tipf (_(DISPLAY_NAME": Channel message from: %s (%s)"),
 								word[1], hexchat_get_info (ph, "channel"));
 		else
-			tray_set_tipf (_(DISPLAY_NAME": %u new public messages."), tray_pub_count);
+			tray_set_tipf (_(DISPLAY_NAME": %u channel messages."), tray_pub_count);
 	}
 
-	if (prefs.hex_input_balloon_chans && (!prefs.hex_away_omit_alerts || tray_find_away_status () != 1))
-		tray_set_balloonf (word[2], _(DISPLAY_NAME": New public message from: %s (%s)"),
+	if (prefs.hex_input_balloon_chans)
+		tray_set_balloonf (word[2], _("Channel message from: %s (%s)"),
 								 word[1], hexchat_get_info (ph, "channel"));
 
 	return HEXCHAT_EAT_NONE;
@@ -762,8 +768,8 @@ tray_priv (char *from, char *text)
 		tray_set_tipf (_(DISPLAY_NAME": %u private messages, latest from: %s (%s)"),
 							tray_priv_count, from, network);
 
-	if (prefs.hex_input_balloon_priv && (!prefs.hex_away_omit_alerts || tray_find_away_status () != 1))
-		tray_set_balloonf (text, _(DISPLAY_NAME": Private message from: %s (%s)"),
+	if (prefs.hex_input_balloon_priv)
+		tray_set_balloonf (text, _("Private message from: %s (%s)"),
 								 from, network);
 }
 
@@ -773,7 +779,7 @@ tray_priv_cb (char *word[], void *userdata)
 	/*if (tray_status == TS_HIGHLIGHT)
 		return HEXCHAT_EAT_NONE;*/
 
-	if (prefs.hex_input_tray_priv && (!prefs.hex_away_omit_alerts || tray_find_away_status () != 1))
+	if (prefs.hex_input_tray_priv)
 		tray_priv (word[1], word[2]);
 
 	return HEXCHAT_EAT_NONE;
@@ -817,7 +823,7 @@ tray_dcc_cb (char *word[], void *userdata)
 	}
 
 	if (prefs.hex_input_balloon_priv && (!prefs.hex_away_omit_alerts || tray_find_away_status () != 1))
-		tray_set_balloonf ("", _(DISPLAY_NAME": File offer from: %s (%s)"),
+		tray_set_balloonf ("", _("File offer from: %s (%s)"),
 								word[1], network);
 
 	return HEXCHAT_EAT_NONE;
@@ -853,7 +859,7 @@ tray_apply_setup (void)
 	}
 	else
 	{
-		if (prefs.hex_gui_tray && !hextray_mode () && !unity_mode ())
+		if (prefs.hex_gui_tray && !unity_mode ())
 			tray_init ();
 	}
 }
@@ -885,7 +891,7 @@ tray_plugin_init (hexchat_plugin *plugin_handle, char **plugin_name,
 
 	hexchat_hook_print (ph, "Focus Window", -1, tray_focus_cb, NULL);
 
-	if (prefs.hex_gui_tray && !hextray_mode () && !unity_mode ())
+	if (prefs.hex_gui_tray && !unity_mode ())
 		tray_init ();
 
 	return 1;       /* return 1 for success */
