@@ -22,16 +22,6 @@
 
 #include "fe-gtk.h"
 
-#include <gtk/gtkbox.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtkdnd.h>
-#include <gtk/gtkentry.h>
-#include <gtk/gtktreeview.h>
-#include <gtk/gtktreeselection.h>
-#include <gtk/gtkscrolledwindow.h>
-#include <gtk/gtkcellrendererpixbuf.h>
-#include <gtk/gtkcellrenderertext.h>
-#include <gtk/gtkliststore.h>
 #include <gdk/gdkkeysyms.h>
 
 #include "../common/hexchat.h"
@@ -50,11 +40,6 @@
 #include "userlistgui.h"
 #include "fkeys.h"
 
-#ifdef USE_GTKSPELL
-#include <gtk/gtktextview.h>
-#endif
-
-
 enum
 {
 	COL_PIX=0,		// GdkPixbuf *
@@ -335,7 +320,7 @@ fe_userlist_rehash (session *sess, struct User *user)
 	if (!iter)
 		return;
 
-	if (prefs.hex_away_track && prefs.hex_away_size_max && user->away)
+	if (prefs.hex_away_track && user->away)
 		nick_color = COL_AWAY;
 	else if (prefs.hex_gui_ulist_color)
 		nick_color = text_color_of(user->nick);
@@ -355,7 +340,7 @@ fe_userlist_insert (session *sess, struct User *newuser, int row, int sel)
 	char *nick;
 	int nick_color = 0;
 
-	if (prefs.hex_away_track && prefs.hex_away_size_max && newuser->away)
+	if (prefs.hex_away_track && newuser->away)
 		nick_color = COL_AWAY;
 	else if (prefs.hex_gui_ulist_color)
 		nick_color = text_color_of(newuser->nick);
