@@ -32,7 +32,6 @@
 #endif
 #include <sys/types.h>
 #include <ctype.h>
-#include <glib.h>
 #include <glib-object.h>
 #include "../common/hexchat.h"
 #include "../common/hexchatc.h"
@@ -466,8 +465,9 @@ static const GOptionEntry gopt_entries[] =
  {"no-plugins",	'n', 0, G_OPTION_ARG_NONE,	&arg_skip_plugins, N_("Don't auto load any plugins"), NULL},
  {"plugindir",	'p', 0, G_OPTION_ARG_NONE,	&arg_show_autoload, N_("Show plugin/script auto-load directory"), NULL},
  {"configdir",	'u', 0, G_OPTION_ARG_NONE,	&arg_show_config, N_("Show user config directory"), NULL},
- {"url",	 0,  0, G_OPTION_ARG_STRING,	&arg_url, N_("Open an irc://server:port/channel URL"), "URL"},
+ {"url",	 0,  G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_STRING,	&arg_url, N_("Open an irc://server:port/channel URL"), "URL"},
  {"version",	'v', 0, G_OPTION_ARG_NONE,	&arg_show_version, N_("Show version information"), NULL},
+ {G_OPTION_REMAINING, '\0', 0, G_OPTION_ARG_STRING_ARRAY, &arg_urls, N_("Open an irc://server:port/channel?key URL"), "URL"},
  {NULL}
 };
 
@@ -486,8 +486,6 @@ fe_args (int argc, char *argv[])
 	context = g_option_context_new (NULL);
 	g_option_context_add_main_entries (context, gopt_entries, GETTEXT_PACKAGE);
 	g_option_context_parse (context, &argc, &argv, &error);
-	
-	g_type_init ();
 
 	if (error)
 	{
@@ -517,7 +515,7 @@ fe_args (int argc, char *argv[])
 		}
 		free (exe);
 #else
-		printf ("%s\n", HEXCHATLIBDIR"/plugins");
+		printf ("%s\n", HEXCHATLIBDIR);
 #endif
 		return 0;
 	}
